﻿using System;
using System.IO.Ports;
using System.Threading;

namespace NixieClockSync
{
    class Program
    {
        static SerialPort _p;
        static void Main(string[] args)
        {
            // можно задать порт аргументом
            _p = new SerialPort(args.Length == 1 ? args[0] : "COM12", 9600, Parity.None, 8, StopBits.One) { ReadTimeout = 50000, WriteTimeout = 50000, NewLine = "\n" };
            _p.Open();
            try
            {
                if (!WritePort("t", 1))
                {
                    Console.WriteLine("Nixie not found");
                    return;
                }
                Console.WriteLine("Nixie connected");
                SetTime();

                var calibrationSecond = 1000;
                SetSecond(calibrationSecond);

                const int calLen = 200; // время калибровки в секундах, чем больше - тем точнее. 200 самый раз

                var span = Calibrate(calLen);
                calibrationSecond = (int)Math.Round(calibrationSecond * calLen * 1000.0 / span.TotalMilliseconds);
                if (calibrationSecond < 100 || calibrationSecond > 2000)
                {
                    throw new Exception("Calibration error (calibrationSecond=" + calibrationSecond + ")");
                }
                SetSecond(calibrationSecond);
                SetTime();
                span = Calibrate(calLen);
                Console.WriteLine("Test ms is " + (calLen * 1000) + ", real is " + span.TotalMilliseconds);
            }
            catch (Exception e) { Console.WriteLine(e); }
            if (_p.IsOpen) _p.Close();
        }

        static void SetSecond(int millis)
        {
            Thread.Sleep(1000);
            Console.Write("Change second to " + millis + "ms...");
            Console.WriteLine(WritePort("s", millis) ? "Done." : "Error!");
        }
        static TimeSpan Calibrate(int calibrationTimeSeconds)
        {
            Thread.Sleep(500);
            _p.ReadTimeout = 1000 * calibrationTimeSeconds * 2;
            if (!WritePort("c", calibrationTimeSeconds)) return new TimeSpan(0);
            var dt1 = DateTime.Now;
            var line = _p.ReadLine().Trim();
            if (line == "st")
                dt1 = DateTime.Now;
            Console.WriteLine("Calibration start time: " + dt1.ToString("T"));
            line = _p.ReadLine().Trim();
            if (line == "en")
            {
                var dt2 = DateTime.Now;
                Console.WriteLine("Calibration end time: " + dt2.ToString("T"));
                return dt2 - dt1;
            }
            return new TimeSpan(0);
        }

        static bool WritePort(string cmd, int prm)
        {
            Console.BackgroundColor = ConsoleColor.DarkGreen;
            int chet = prm % 255;
            Console.WriteLine("--> {0}{1},{3}, chet={2} ({3})", cmd, prm, chet, chet);
            _p.Write(cmd + prm + "," + chet);
            _p.BaseStream.Flush();
            Thread.Sleep(50);
            var line = _p.ReadTo("\n").Trim();
            Console.WriteLine("<-- " + line);
            if (line.StartsWith("ERROR")) return WritePort(cmd, prm);
            var r = line == "OK" + cmd + prm;
            Console.WriteLine(r ? "OK" : "ERROR");
            Console.BackgroundColor = ConsoleColor.Black;
            return r;
        }

        static void SetTime()
        {
            Console.WriteLine("Setting time...");
            WritePort("h", DateTime.Now.Hour);
            WritePort("m", DateTime.Now.Minute);
            Console.WriteLine("Done.");
        }
    }
}
